# -*- coding: iso_8859_1 -*-

import numpy as np
from scipy.optimize import leastsq

#----------------------------------------------------------------------------------------
#def fit_gauss_circular(xy, data):
#def fit_gauss_elliptical(xy, data):
#def fit_moffat_circular(xy, data):
#def fit_moffat_elliptical(xy, data):
#
#--- purpose ---
#these functions perform a 2D PSF fitting on some stellar object, using a circular or elliptical Gaussian or Moffat PSF model 
#----------------------------------------------------------------------------------------

def fit_gauss_circular(xy, data):

	#find starting values
	maxi = data.max()
	floor = np.ma.median(data.flatten())
	height = maxi - floor

	mean_x = (np.shape(data)[0]-1)/2
	mean_y = (np.shape(data)[1]-1)/2

	fwhm = np.sqrt(np.sum((data>floor+height/2.).flatten()))
	
	#---------------------------------------------------------------------------------
	sig = fwhm / (2.*np.sqrt(2.*np.log(2.)))
	width = 0.5/np.square(sig)
	
	p0 = floor, height, mean_x, mean_y, width

	#---------------------------------------------------------------------------------
	#fitting gaussian
	def gauss(floor, height, mean_x, mean_y, width):		
		return lambda x,y: floor + height*np.exp(-np.abs(width)*((x-mean_x)**2+(y-mean_y)**2))

	def err(p,data):
		return np.ravel(gauss(*p)(*np.indices(data.shape))-data)
	
	p = leastsq(err, p0, args=(data), maxfev=1000)
	p = p[0]
	
	#---------------------------------------------------------------------------------
	#formatting results
	floor = p[0]
	height = p[1]
	mean_x = p[2] + xy[0]
	mean_y = p[3] + xy[1]

	sig = np.sqrt(0.5/np.abs(p[4]))
	fwhm = sig * (2.*np.sqrt(2.*np.log(2.)))	
	
	output = [maxi, floor, height, mean_x, mean_y, fwhm]
	return output

def fit_gauss_elliptical(xy, data):

	#find starting values
	maxi = data.max()
	floor = np.ma.median(data.flatten())
	height = maxi - floor

	mean_x = (np.shape(data)[0]-1)/2
	mean_y = (np.shape(data)[1]-1)/2

	fwhm = np.sqrt(np.sum((data>floor+height/2.).flatten()))
	fwhm_x = fwhm
	fwhm_y = fwhm
	sig_x = fwhm_x / (2.*np.sqrt(2.*np.log(2.)))
	sig_y = fwhm_y / (2.*np.sqrt(2.*np.log(2.)))	
	
	angle = 0.

	p0 = floor, height, mean_x, mean_y, sig_x, sig_y, angle

	#---------------------------------------------------------------------------------
	#fitting gaussian
	def gauss(floor, height, mean_x, mean_y, sig_x, sig_y, angle):
		
		A = (np.cos(angle)/sig_x)**2. + (np.sin(angle)/sig_y)**2.
		B = (np.sin(angle)/sig_x)**2. + (np.cos(angle)/sig_y)**2.
		C = 2.0*np.sin(angle)*np.cos(angle)*(1./(sig_x**2.)-1./(sig_y**2.))

		#do not forget factor 0.5 in exp(-0.5*r**2./sig**2.)	
		return lambda x,y: floor + height*np.exp(-0.5*(A*((x-mean_x)**2)+B*((y-mean_y)**2)+C*(x-mean_x)*(y-mean_y)))

	def err(p,data):
		return np.ravel(gauss(*p)(*np.indices(data.shape))-data)
	
	p = leastsq(err, p0, args=(data), maxfev=1000)
	p = p[0]
	
	#---------------------------------------------------------------------------------
	#formatting results
	floor = p[0]
	height = p[1]
	mean_x = p[2] + xy[0]
	mean_y = p[3] + xy[1]
	
	#angle gives the direction of the p[4]=sig_x axis, starting from x (vertical) axis, clockwise in direction of y (horizontal) axis
	if np.abs(p[4])>np.abs(p[5]):

		fwhm_large = np.abs(p[4]) * (2.*np.sqrt(2.*np.log(2.)))
		fwhm_small = np.abs(p[5]) * (2.*np.sqrt(2.*np.log(2.)))	
		angle = np.arctan(np.tan(p[6]))
			
	else:	#then sig_x is the smallest : we want angle to point to sig_y, the largest
	
		fwhm_large = np.abs(p[5]) * (2.*np.sqrt(2.*np.log(2.)))
		fwhm_small = np.abs(p[4]) * (2.*np.sqrt(2.*np.log(2.)))	
		angle = np.arctan(np.tan(p[6]+np.pi/2.))
	
	output = [maxi, floor, height, mean_x, mean_y, fwhm_small, fwhm_large, angle]
	return output

def fit_moffat_circular(xy, data):

	#---------------------------------------------------------------------------------
	#find starting values
	maxi = data.max()
	floor = np.ma.median(data.flatten())
	height = maxi - floor

	mean_x = (np.shape(data)[0]-1)/2
	mean_y = (np.shape(data)[1]-1)/2

	fwhm = np.sqrt(np.sum((data>floor+height/2.).flatten()))

	beta = 4
	
	p0 = floor, height, mean_x, mean_y, fwhm, beta

	#---------------------------------------------------------------------------------
	#fitting gaussian
	def moffat(floor, height, mean_x, mean_y, fwhm, beta):
		sig = 0.5*fwhm/np.sqrt(2.**(1./beta)-1.)	
		return lambda x,y: floor + height/((1.+(((x-mean_x)**2+(y-mean_y)**2)/sig**2.))**beta)

	def err(p,data):
		return np.ravel(moffat(*p)(*np.indices(data.shape))-data)
	
	p = leastsq(err, p0, args=(data), maxfev=1000)
	p = p[0]
	
	#---------------------------------------------------------------------------------
	#formatting results
	floor = p[0]
	height = p[1]
	mean_x = p[2] + xy[0]
	mean_y = p[3] + xy[1]
	fwhm = np.abs(p[4])
	beta = p[5]
	
	output = [maxi, floor, height, mean_x, mean_y, fwhm, beta]
	return output

def fit_moffat_elliptical(xy, data):

	#---------------------------------------------------------------------------------
	#find starting values
	maxi = data.max()
	floor = np.ma.median(data.flatten())
	height = maxi - floor

	mean_x = (np.shape(data)[0]-1)/2
	mean_y = (np.shape(data)[1]-1)/2

	fwhm = np.sqrt(np.sum((data>floor+height/2.).flatten()))
	fwhm_x = fwhm
	fwhm_y = fwhm

	angle = 0.
	beta = 4
	
	p0 = floor, height, mean_x, mean_y, fwhm_x, fwhm_y, angle, beta

	#---------------------------------------------------------------------------------
	#fitting gaussian
	def moffat(floor, height, mean_x, mean_y, fwhm_x, fwhm_y, angle, beta):
		
		sig_x = 0.5*fwhm_x/np.sqrt(2.**(1./beta)-1.)
		sig_y = 0.5*fwhm_y/np.sqrt(2.**(1./beta)-1.)
	
		A = (np.cos(angle)/sig_x)**2. + (np.sin(angle)/sig_y)**2.
		B = (np.sin(angle)/sig_x)**2. + (np.cos(angle)/sig_y)**2.
		C = 2.0*np.sin(angle)*np.cos(angle)*(1./sig_x**2. - 1./sig_y**2.)
		
		return lambda x,y: floor + height/((1.+ A*((x-mean_x)**2) + B*((y-mean_y)**2) + C*(x-mean_x)*(y-mean_y))**beta)

	def err(p,data):
		return np.ravel(moffat(*p)(*np.indices(data.shape))-data)
	
	p = leastsq(err, p0, args=(data), maxfev=1000)
	p = p[0]
	
	#---------------------------------------------------------------------------------
	#formatting results
	floor = p[0]
	height = p[1]
	mean_x = p[2] + xy[0]
	mean_y = p[3] + xy[1]
	beta = p[7]
	
	#angle gives the direction of the p[4]=sig_x axis, starting from x (vertical) axis, clockwise in direction of y (horizontal) axis
	if np.abs(p[4])>np.abs(p[5]):

		fwhm_large = np.abs(p[4])
		fwhm_small = np.abs(p[5])
		angle = np.arctan(np.tan(p[6]))
			
	else:	#then sig_x is the smallest : we want angle to point to sig_y, the largest
	
		fwhm_large = np.abs(p[5])
		fwhm_small = np.abs(p[4])
		angle = np.arctan(np.tan(p[6]+np.pi/2.))

	output = [maxi, floor, height, mean_x, mean_y, fwhm_small, fwhm_large, angle, beta]
	return output
