# -*- coding: iso_8859_1 -*-

import numpy as np
import sys
import process, stats, transform
import astropython.conf as conf
import scipy.fftpack as fft

def find_threshold(image0, detection_area, nb_stars, excluded=[], included=[]):
	
	th0 = 0.3		#be careful here, some combinations of values must be avoided, such as 0.4 here together with 1.5 as scaling factor (then infinite loops are possible)
	nb0 = len(find_stars(image0, th0, detection_area, excluded=excluded, included=included))
	
	while True:
		if nb0>nb_stars*1.5:
			th0 = 1. - (1.-th0)/1.5
			nb0 = len(find_stars(image0, th0, detection_area, excluded=excluded, included=included))

		elif nb0<nb_stars*0.5:
			th0 = th0 / 1.5
			nb0 = len(find_stars(image0, th0, detection_area, excluded=excluded, included=included))

		else:
			break

	return th0

#----------------------------------------------------------------------------------------
#def find_stars(image0, threshold, detection_area, excluded=[], included=[]):
#
#--- purpose ---
#detects the positions of the brightest stars on one image
#stars very close to image boundaries are ignored
#----------------------------------------------------------------------------------------

def find_stars(image0, threshold, detection_area, excluded=[], included=[]):

	#image sizes
	size_x = image0.shape[0]
	size_y = image0.shape[1]

	#calculating threshold
	mean = np.mean(image0.flatten())
	#mean = stats.get_median(image0)[0]		#less prone to errors but slower
	maxi = np.max(image0.flatten())
	threshold = mean + threshold*(maxi-mean)
	
	#finding potential star areas
	indices = np.where(image0>threshold)
	x = indices[0]
	y = indices[1]
	
	#finding approximate star centers
	stars1 = []
	for i in range(0,len(x)):
		if (x[i]>=detection_area)&(x[i]<size_x-detection_area)&(y[i]>=detection_area)&(y[i]<size_y-detection_area):
			area = image0[x[i]-detection_area:x[i]+detection_area+1,y[i]-detection_area:y[i]+detection_area+1]
			xcenter,ycenter = np.where(area==np.max(np.max(area)))
			stars1.append([xcenter[0]+x[i]-detection_area, ycenter[0]+y[i]-detection_area])

	#removes hot pixels and double occurrences
	stars2 = []
	for center in stars1:
		stars1.remove(center)
		if center in stars1:
			if center not in stars2:
				stars2.append(center)

	#removes saturated stars
	stars3 = stars2
	#stars3 = []
	#for xy in stars2:
	#	if image0[xy[0],xy[1]]!=conf.saturation:
	#		stars3.append(xy)
	
	#excluded or included area
	stars4 = []
	for xy in stars3:
		test = 1
		if len(included)>0:
			test = 0
			for zone in included:
				xmin, xmax, ymin, ymax = zone
				if (xy[0]>=xmin)and(xy[0]<=xmax)and(xy[1]>=ymin)and(xy[1]<=ymax):
					test = 1
		elif len(excluded)>0:
			for zone in excluded:
				xmin, xmax, ymin, ymax = zone
				if (xy[0]>=xmin)and(xy[0]<=xmax)and(xy[1]>=ymin)and(xy[1]<=ymax):
					test = 0
		if test==1:
			stars4.append(xy)

	return stars4

def find_stars_and_more(image0, threshold, detection_area, excluded=[], included=[], full_info=False):

	#image sizes
	size_x = image0.shape[0]
	size_y = image0.shape[1]
				
	#finding accurate star centers
	stars4 = find_stars(image0, threshold, detection_area, excluded=excluded, included=included)

	stars5 = []
	for xy in stars4:
		#print xy
		if (xy[0]>=detection_area)&(xy[0]<size_x-detection_area)&(xy[1]>=detection_area)&(xy[1]<size_y-detection_area):
			p = process.fit_gauss_elliptical([xy[0]-detection_area,xy[1]-detection_area],image0[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1])
			max, floor, height, mean_x, mean_y, fwhm_1, fwhm_2, angle = p
			if full_info:
				stars5.append([max, floor, height, mean_x, mean_y, fwhm_1, fwhm_2, angle])			
			else:
				stars5.append([mean_x, mean_y, max])

	#ordering by decreasing intensity
	stars5 = np.array(stars5)
	if stars5.shape[0]>0:
		stars5 = np.flipud(stars5[np.argsort(stars5[:,2]),:])
		
	return stars5

#----------------------------------------------------------------------------------------
#def find_shift_stars_matching(image1, image2, threshold, detection_area, excluded=[], included=[], text_display=False):
#
#--- purpose ---
#calculates the non-integer x,y shift between two images of the same star field
#algorithm reports the most likely shift, that allows the greatest number of stars to coincide
#----------------------------------------------------------------------------------------

def find_shift_stars_matching(image1, image2, threshold, detection_area, excluded=[], included=[], text_display=False):

	step = 1.			#shift difference value below which shifts are assumed to belong to the same translation scenario
	#-------------------------------------------------------------------------------------
	#	finding stars
	#-------------------------------------------------------------------------------------

	stars1 = find_stars_and_more(image1, threshold, detection_area, excluded, included)
	stars2 = find_stars_and_more(image2, threshold, detection_area, excluded, included)
	
	if (stars1.shape[0]==0)or(stars2.shape[0]==0):
		raw_input('no stars could be detected, shift will not be calculated.\nPress Enter to close')
		sys.exit()		

	#-------------------------------------------------------------------------------------
	#	looking for x and y shift
	#-------------------------------------------------------------------------------------

	delta_x = np.tile(stars2[:,0],(len(stars1),1)) - np.tile(np.reshape(stars1[:,0],(-1,1)),(1,len(stars2)))
	delta_x = delta_x.flatten()

	delta_y = np.tile(stars2[:,1],(len(stars1),1)) - np.tile(np.reshape(stars1[:,1],(-1,1)),(1,len(stars2)))
	delta_y = delta_y.flatten()

	nb_x = np.tile(delta_x,(len(delta_x),1)) - np.tile(np.reshape(delta_x,(-1,1)),(1,len(delta_x)))
	nb_y = np.tile(delta_y,(len(delta_y),1)) - np.tile(np.reshape(delta_y,(-1,1)),(1,len(delta_y)))
	nb = (np.abs(nb_x)<=step) * (np.abs(nb_y)<=step)
	nb = np.sum(nb,axis=1)
	nb_max = np.max(nb)

	if text_display:
		print 'shift comparison result ='
		print nb

	#if list(nb).count(nb_max) != nb_max:
	#	raw_input('more than one solution, shift could not be calculated.\nPress Enter to close')
	#	sys.exit()
	
	dx = [delta_x[i] for i in range(len(delta_x)) if nb[i]>=nb_max-1]
	dy = [delta_y[i] for i in range(len(delta_y)) if nb[i]>=nb_max-1]

	return [np.mean(dx), np.mean(dy)]

#----------------------------------------------------------------------------------------
#def find_shift_crosscorrelation(image1, image2):
#
#--- purpose ---
#calculates the non-integer x,y shift between two images of the same star field
#algorithm is based on the cross-correlation of the two images
#if there is a moving object, it cannot be disregarded during the calculation, making this technique less accurate
#----------------------------------------------------------------------------------------

def find_shift_crosscorrelation(image1, image2):

	#--- calculates cross-correlation between two images ---
	im1 = fft.fft2(image1)
	im2 = fft.fft2(image2)
	crosscorr = fft.ifft2(im1*np.ma.conjugate(im2))

	#--- finds the position of the max of the modulus of the cross-correlation ---
	modulus = fft.fftshift(np.abs(crosscorr))
	dx, dy = np.where(modulus==np.max(modulus.flatten()))
			
	#--- zoom in on max position with scaling factor "size", finds again the position of the max with better accuracy ---
	size = 100
	submod = modulus[dx-size:dx+size+1,dy-size:dy+size+1]
	submod2 = transform.scale_Bspline(submod, [size, size], 100., 3)
	dx2, dy2 = np.where(submod2==np.max(submod2.flatten()))

	#--- combines previous results
	dx3 = image1.shape[0]/2. - np.float(dx+(dx2-100.)/100.)
	dy3 = image1.shape[1]/2. - np.float(dy+(dy2-100.)/100.)
	return [dx3, dy3]
